@@ -177,8 +177,8 @@ class PostgreSQLAdapter(Adapter):
             conn.set_isolation_level(isolation)
             cursor = conn.cursor()
             cursor.arraysize = 64
-        except psycopg2.OperationalError:
-            log.warning("Unable to connect in %s", repr(self))
+        except psycopg2.OperationalError, e:
+            log.warning("Unable to connect: %s", e)
             raise
         return conn, cursor
 
@@ -211,9 +211,11 @@ class PostgreSQLAdapter(Adapter):
         return conn, cursor
 
     def restart_load(self, cursor):
-        """After a rollback, reinitialize a connection for loading objects."""
-        # No re-init necessary
-        pass
+        """Reinitialize a connection for loading objects."""
+        try:
+            cursor.connection.rollback()
+        except (psycopg2.OperationalError, psycopg2.InterfaceError), e:
+            raise StorageError(e)
 
     def get_object_count(self):
         """Returns the number of objects in the database"""
@@ -367,8 +369,8 @@ class PostgreSQLAdapter(Adapter):
                 cursor.connection.set_isolation_level(
                     psycopg2.extensions.ISOLATION_LEVEL_READ_COMMITTED)
             self._make_temp_table(cursor)
-        except (psycopg2.OperationalError, psycopg2.InterfaceError):
-            raise StorageError("database disconnected")
+        except (psycopg2.OperationalError, psycopg2.InterfaceError), e:
+            raise StorageError(e)
 
     def store_temp(self, cursor, oid, prev_tid, md5sum, data):
         """Store an object in the temporary table."""
@@ -573,55 +575,4 @@ class PostgreSQLAdapter(Adapter):
         # No action needed
         pass
 
-
-    def poll_invalidations(self, conn, cursor, prev_polled_tid, ignore_tid):
-        """Polls for new transactions.
-
-        conn and cursor must have been created previously by open_for_load().
-        prev_polled_tid is the tid returned at the last poll, or None
-        if this is the first poll.  If ignore_tid is not None, changes
-        committed in that transaction will not be included in the list
-        of changed OIDs.
-
-        Returns (changed_oids, new_polled_tid).  Raises StorageError
-        if the database has disconnected.
-        """
-        try:
-            # find out the tid of the most recent transaction.
-            cursor.execute("EXECUTE get_latest_tid")
-            # Expect the transaction table to always have at least one row.
-            assert cursor.rowcount == 1
-            new_polled_tid = cursor.fetchone()[0]
-
-            if prev_polled_tid is None:
-                # This is the first time the connection has polled.
-                return None, new_polled_tid
-
-            if new_polled_tid == prev_polled_tid:
-                # No transactions have been committed since prev_polled_tid.
-                return (), new_polled_tid
-
-            stmt = "SELECT 1 FROM transaction WHERE tid = %s"
-            cursor.execute(stmt, (prev_polled_tid,))
-            if not cursor.rowcount:
-                # Transaction not found; perhaps it has been packed.
-                # The connection cache needs to be cleared.
-                return None, new_polled_tid
-
-            # Get the list of changed OIDs and return it.
-            stmt = """
-            SELECT DISTINCT zoid
-            FROM object_state
-                JOIN transaction USING (tid)
-            WHERE tid > %s
-            """
-            if ignore_tid is not None:
-                stmt += " AND tid != %d" % ignore_tid
-            cursor.execute(stmt, (prev_polled_tid,))
-            oids = [oid for (oid,) in cursor]
-
-            return oids, new_polled_tid
-
-        except (psycopg2.OperationalError, psycopg2.InterfaceError):
-            raise StorageError("database disconnected")
-
+    _poll_query = "EXECUTE get_latest_tid"
